/* tslint:disable */
/* eslint-disable */
/**
 * Ampli - A strong typed wrapper for your Analytics
 *
 * This file is generated by Amplitude.
 * To update run 'ampli pull integration-functions'
 *
 * Required dependencies: @amplitude/analytics-node@^0.5.0
 * Tracking Plan Version: 158
 * Build: 1.0.0
 * Runtime: node.js:typescript-ampli-v2
 *
 * [View Tracking Plan](https://data.amplitude.com/netlify-1232/Netlify/events/main/latest)
 *
 * [Full Setup Instructions](https://data.amplitude.com/netlify-1232/Netlify/implementation/integration-functions)
 */

import * as amplitude from '@amplitude/analytics-node';

export type NodeClient = amplitude.Types.NodeClient;
export type BaseEvent = amplitude.Types.BaseEvent;
export type Event = amplitude.Types.Event;
export type EventOptions = amplitude.Types.EventOptions;
export type Result = amplitude.Types.Result;
export type NodeOptions = amplitude.Types.NodeOptions;

export type Environment = 'production' | 'development';

export const ApiKey: Record<Environment, string> = {
  production: 'cebfd07e4b8bce591224aa2ab1e3e096',
  development: '60c433cd2d7a65ed140f33f250a840ea'
};

/**
* Default Amplitude configuration options. Contains tracking plan information.
*/
export const DefaultConfiguration: NodeOptions = {
  plan: {
    version: '158',
    branch: 'main',
    source: 'integration-functions',
    versionId: '6bcddd98-85a5-41c2-881c-886939288c3e'
  },
  ...{
    ingestionMetadata: {
      sourceName: 'node.js-typescript-ampli',
      sourceVersion: '2.0.0'
    }
  }
};

export interface LoadOptionsBase { disabled?: boolean }

export type LoadOptionsWithEnvironment = LoadOptionsBase & { environment: Environment; client?: { configuration?: NodeOptions; }; };
export type LoadOptionsWithApiKey = LoadOptionsBase & { client: { apiKey: string; configuration?: NodeOptions; } };
export type LoadOptionsWithClientInstance = LoadOptionsBase & { client: { instance: NodeClient; } };

export type LoadOptions = LoadOptionsWithEnvironment | LoadOptionsWithApiKey | LoadOptionsWithClientInstance;

export interface IntegrationEmailSendProperties {
  /**
   * provider used for signup; will either be a git provider or "email" for email and password
   */
  provider: string;
}

export interface IntegrationSentryCronMonitoringEnabledProperties {
  functionName: string;
}

export interface IntegrationSentryEnabledProperties {
  functionName: string;
}

export class IntegrationEmailSend implements BaseEvent {
  event_type = 'integration:email-send';

  constructor(
    public event_properties: IntegrationEmailSendProperties,
  ) {
    this.event_properties = event_properties;
  }
}

export class IntegrationSentryCronMonitoringEnabled implements BaseEvent {
  event_type = 'integration:sentry_cron_monitoring_enabled';

  constructor(
    public event_properties: IntegrationSentryCronMonitoringEnabledProperties,
  ) {
    this.event_properties = event_properties;
  }
}

export class IntegrationSentryEnabled implements BaseEvent {
  event_type = 'integration:sentry_enabled';

  constructor(
    public event_properties: IntegrationSentryEnabledProperties,
  ) {
    this.event_properties = event_properties;
  }
}

export type PromiseResult<T> = { promise: Promise<T | void> };

const getVoidPromiseResult = () => ({ promise: Promise.resolve() });

// prettier-ignore
export class Ampli {
  private disabled: boolean = false;
  private amplitude?: NodeClient;

  get client(): NodeClient {
    this.isInitializedAndEnabled();
    return this.amplitude!;
  }

  get isLoaded(): boolean {
    return this.amplitude != null;
  }

  private isInitializedAndEnabled(): boolean {
    if (!this.amplitude) {
      console.error('ERROR: Ampli is not yet initialized. Have you called ampli.load() on app start?');
      return false;
    }
    return !this.disabled;
  }

  /**
   * Initialize the Ampli SDK. Call once when your application starts.
   *
   * @param options Configuration options to initialize the Ampli SDK with.
   */
  load(options: LoadOptions): PromiseResult<void> {
    this.disabled = options.disabled ?? false;

    if (this.amplitude) {
      console.warn('WARNING: Ampli is already initialized. Ampli.load() should be called once at application startup.');
      return getVoidPromiseResult();
    }

    let apiKey: string | null = null;
    if (options.client && 'apiKey' in options.client) {
      apiKey = options.client.apiKey;
    } else if ('environment' in options) {
      apiKey = ApiKey[options.environment];
    }

    if (options.client && 'instance' in options.client) {
      this.amplitude = options.client.instance;
    } else if (apiKey) {
      this.amplitude = amplitude.createInstance();
      return this.amplitude.init(apiKey, { ...DefaultConfiguration, ...options.client?.configuration });
    } else {
      console.error("ERROR: ampli.load() requires 'environment', 'client.apiKey', or 'client.instance'");
    }

    return getVoidPromiseResult();
  }

  /**
   * Identify a user and set user properties.
   *
   * @param userId The user's id.
   * @param options Optional event options.
   */
  identify(
    userId: string | undefined,
    options?: EventOptions,
  ): PromiseResult<Result> {
    if (!this.isInitializedAndEnabled()) {
      return getVoidPromiseResult();
    }

    if (userId) {
      options = {...options,  user_id: userId};
    }

    const amplitudeIdentify = new amplitude.Identify();

    return this.amplitude!.identify(amplitudeIdentify, options);
  }

  /**
   * Set Group for the user.
   *
   * @param userId The user's id.
   * @param groupType The group type.
   * @param groupName The group name.
   * @param options Optional event options.
   */
  setGroup(
    userId: string | undefined,
    groupType: string,
    groupName: string | string[],
    options?: EventOptions,
  ): PromiseResult<Result> {
    if (!this.isInitializedAndEnabled()) {
      return getVoidPromiseResult();
    }

    if (userId) {
      options = {...options,  user_id: userId};
    }

    return this.amplitude!.setGroup(groupType, groupName, options);
  }

  /**
   * Identify a group and set group properties.
   *
   * @param groupType The group type.
   * @param groupName The group name.
   * @param options Optional event options.
   */
  groupIdentify(
    groupType: string,
    groupName: string | string[],
    options?: EventOptions,
  ): PromiseResult<Result> {
    if (!this.isInitializedAndEnabled()) {
      return getVoidPromiseResult();
    }

    const amplitudeIdentify = new amplitude.Identify();

    return this.amplitude!.groupIdentify(groupType, groupName, amplitudeIdentify, options);
  }

  /**
   * Track event
   *
   * @param userId The user's id.
   * @param event The event to track.
   * @param options Optional event options.
   */
  track(userId: string | undefined, event: Event, options?: EventOptions): PromiseResult<Result> {
    if (!this.isInitializedAndEnabled()) {
      return getVoidPromiseResult();
    }

    if (userId) {
      options = {...options,  user_id: userId};
    }

    return this.amplitude!.track(event, undefined, options);
  }

  flush(): PromiseResult<void> {
    if (!this.isInitializedAndEnabled()) {
      return getVoidPromiseResult();
    }

    return this.amplitude!.flush();
  }

  /**
   * integration:email-send
   *
   * [View in Tracking Plan](https://data.amplitude.com/netlify-1232/Netlify/events/main/latest/integration%3Aemail-send)
   *
   * Owner: Ed Stephinson
   *
   * @param userId The user's ID.
   * @param properties The event's properties (e.g. provider)
   * @param options Amplitude event options.
   */
  integrationEmailSend(
    userId: string | undefined,
    properties: IntegrationEmailSendProperties,
    options?: EventOptions,
  ) {
    return this.track(userId, new IntegrationEmailSend(properties), options);
  }

  /**
   * integration:sentry_cron_monitoring_enabled
   *
   * [View in Tracking Plan](https://data.amplitude.com/netlify-1232/Netlify/events/main/latest/integration%3Asentry_cron_monitoring_enabled)
   *
   * Tracks whether Sentry's Cron Monitoring feature is enabled for a Netlify Function. This should realistically only be used by Netlify Scheduled Functions
   *
   * Owner: erica.pisani@netlify.com
   *
   * @param userId The user's ID.
   * @param properties The event's properties (e.g. functionName)
   * @param options Amplitude event options.
   */
  integrationSentryCronMonitoringEnabled(
    userId: string | undefined,
    properties: IntegrationSentryCronMonitoringEnabledProperties,
    options?: EventOptions,
  ) {
    return this.track(userId, new IntegrationSentryCronMonitoringEnabled(properties), options);
  }

  /**
   * integration:sentry_enabled
   *
   * [View in Tracking Plan](https://data.amplitude.com/netlify-1232/Netlify/events/main/latest/integration%3Asentry_enabled)
   *
   * Tracks if Sentry is enabled for a given Netlify Function (Standard, Background, and Scheduled)
   *
   * Owner: erica.pisani@netlify.com
   *
   * @param userId The user's ID.
   * @param properties The event's properties (e.g. functionName)
   * @param options Amplitude event options.
   */
  integrationSentryEnabled(
    userId: string | undefined,
    properties: IntegrationSentryEnabledProperties,
    options?: EventOptions,
  ) {
    return this.track(userId, new IntegrationSentryEnabled(properties), options);
  }
}

export const ampli = new Ampli();
